import unittest
import cffi
import os.path
ffi = cffi.FFI()
#from mock import MagicMock, call


class RFIDTestCase(unittest.TestCase):
    def setUp(self):
        ffi.cdef("""
	        typedef struct {...;} rfid_dev;
	        typedef ... serial_dev;
	        int rfid_init(rfid_dev* d);
	        int serial_tx(serial_dev* d, uint8_t ch);
	        int serial_rx(serial_dev* d, uint8_t* ch);
	        void (*py_serial_tx) (uint8_t ch);
	        uint8_t (*py_serial_rx) (void);
        """)

        self.uut = ffi.verify("""
	        #define MOCKING
	        #include "rfid.c"
	        static void (*py_serial_tx) (uint8_t ch);
	        static uint8_t (*py_serial_rx) (void);
	        int serial_tx(serial_dev* d, uint8_t ch)
	        {
		        py_serial_tx(ch);
		        return 0;
	        }
	        int serial_rx(serial_dev* d, uint8_t* ch)
	        {
		        *ch = py_serial_rx();
		        return 0;
	        }
        """, include_dirs=[os.path.abspath(os.path.dirname(__file__))])

        #self.mock = MagicMock()
        #self.mock.put = MagicMock(name='put')
        #self.mock.get = MagicMock(name='get')
        class TestGet:
            seq = "OK"
            def __call__(self):
                yield seq
        
        class TestPut:
            def __call__(self, ch):
                pass
        
        self.tget = TestGet()
        self.tput = TestPut()
        self.uut.py_serial_tx = ffi.callback('void (*) (uint8_t ch)', self.tput)
        self.uut.py_serial_rx = ffi.callback('uint8_t (*) (void)', self.tget)
		
        #self.uut.py_serial_tx = ffi.callback('void (*) (uint8_t ch)', self.mock.put)
        #self.uut.py_serial_rx = ffi.callback('uint8_t (*) (void)', self.mock.get)

    def test_init(self):
        #self.mock.get.side_effect = ['O','K']
        rfid_dev = ffi.new("rfid_dev*")
        self.uut.rfid_init(rfid_dev)
        #calls = [call('I'), call('N'), call('I'), call('T')]
        #self.mock.put.assert_has_calls(calls)
