from cffi import FFI

CFFI_CDEF = """
    typedef struct TDigest TDigest;

    void random_seed(void);
    TDigest *TDigest_create(double delta, unsigned int K);
    void TDigest_add(TDigest **digest, double x, size_t w);
    double TDigest_percentile(TDigest *digest, double q);
"""


tdigest_ffi = FFI()
tdigest_ffi.cdef(CFFI_CDEF)

with open("tdigest.c") as source:
    tdigest_ffi.set_source("_tdigest", source.read(), include_dirs=["."])

if __name__ == "__main__":
    tdigest_ffi.compile(verbose=True)

    from _tdigest import lib, ffi

    lib.random_seed()
    st = lib.TDigest_create(0.01, 25)
    reference = ffi.new("TDigest **digest", st)

    for i in range(10_000):
        lib.TDigest_add(reference, i, 1)

    print(lib.TDigest_percentile(st, 0.99))
